﻿using DocumentFormat.OpenXml.Wordprocessing;
using Hotline.Api.Controllers;
using Hotline.Application.Identity;
using Hotline.Application.Snapshot;
using Hotline.File;
using Hotline.Identity.Accounts;
using Hotline.Identity.Roles;
using Hotline.Share.Dtos.Article;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Tools;
using Hotline.Snapshot;
using Hotline.Users;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Shouldly;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Application.Tests.Application;
public class SnapshotApplicationTest : TestBase
{
    private readonly ISnapshotApplication _snapshotApplication;
    private readonly IIdentityAppService _identityAppService;
    private readonly IRepository<RedPackRecord> _redPackRecordRepository;
    private readonly IIndustryApplication _industryApplication;
    private readonly IIndustryRepository _industryRepository;
    private readonly IFileRepository _fileRepository;

    public SnapshotApplicationTest(IAccountRepository accountRepository, IRepository<Role> roleRepository, UserController userController, IServiceScopeFactory scopeFactory, IRepository<User> userRepository, IHttpContextAccessor httpContextAccessor, ISnapshotApplication snapshotApplication, IIdentityAppService identityAppService, IRepository<RedPackRecord> redPackRecordRepository, IIndustryApplication industryApplication, IIndustryRepository industryRepository, IFileRepository fileRepository) : base(accountRepository, roleRepository, userController, scopeFactory, userRepository, httpContextAccessor)
    {
        _snapshotApplication = snapshotApplication;
        _identityAppService = identityAppService;
        _redPackRecordRepository = redPackRecordRepository;
        _industryApplication = industryApplication;
        _industryRepository = industryRepository;
        _fileRepository = fileRepository;
    }

    [Fact]
    public async Task GetHomePage_Test()
    {
        var result = await _snapshotApplication.GetHomePageAsync();
        result.Industrys.Any().ShouldBe(true, "首页数据为空");
        result.Industrys.First().DisplayOrder.ShouldBe(1, "排序异常");
    }

    [Fact]
    public async Task GetBulletins_Test()
    {
        var homePage = await _snapshotApplication.GetHomePageAsync();
        var inDto = new BulletinInDto
        {
            IndustryId = homePage.Industrys.First(m => m.Name == "文化旅游").Id,
        };
        var items = await _snapshotApplication.GetBulletinsAsync(inDto);
        items.ShouldNotBeNull();
        items.Any().ShouldBe(true, "公告数据为空");
        items.Any(m => m.Title.IsNullOrEmpty()).ShouldBe(false, "标题错误");
        items.Any(m => m.Content.IsNullOrEmpty()).ShouldBe(false, "内容错误");
        items.Any(m => m.Id.IsNullOrEmpty()).ShouldBe(false, "Id错误");
    }

    [Fact]
    public async Task GetSnapshotUserInfo_Test()
    {
        var result = await _snapshotApplication.GetSnapshotUserInfoAsync();
        result.ShouldNotBeNull();
        result.PhoneNumber.ShouldNotBeNullOrEmpty();
    }

    [Fact]
    public async Task GetThirdToken_Test()
    {
        var result = await _identityAppService.GetThredTokenAsync(new ThirdTokenInDto { LoginCode = "0c3Adhll2zDMBe413rnl2KvEym2AdhlH" });
    }

    [Theory]
    [InlineData("")]
    [InlineData("测")]
    public async Task SnapshotOrder_Test(string key)
    {
        var dto = new OrderInDto();
        dto.KeyWords = key;
        var page = await _snapshotApplication.GetSnapshotOrdersAsync(dto);
        page.Total.ShouldNotBe(0);
        page.Items.FirstOrDefault()?.IndustryName.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.OrderNo.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.StatusText.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.Area.ShouldNotBeNullOrEmpty();
    }

    [Theory]
    [InlineData(EOrderQueryStatus.All, 3)]
    [InlineData(EOrderQueryStatus.Reply, 2)]
    [InlineData(EOrderQueryStatus.NoReply, 1)]
    [InlineData(EOrderQueryStatus.Appraise, 1)]
    public async Task SnapshotOrderStatus_Test(EOrderQueryStatus status, int count)
    {
        var dto = new OrderInDto { Status = status };
        var page = await _snapshotApplication.GetSnapshotOrdersAsync(dto);
        page.Total.ShouldNotBe(0, $"状态:{status.GetDescription()} 数据为空");
        page.Total.ShouldBe(count, $"状态:{status.GetDescription()} 数据条数错误");
        page.Items.FirstOrDefault()?.IndustryName.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.OrderNo.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.StatusText.ShouldNotBeNullOrEmpty();
        page.Items.FirstOrDefault()?.Area.ShouldNotBeNullOrEmpty();

        dto.PageIndex = 2;
        page = await _snapshotApplication.GetSnapshotOrdersAsync(dto);
        page.Items.Count.ShouldBe(0);
    }

    [Fact]
    public async Task GetSnapshotOrderDetail_Test()
    {
        var page = await _snapshotApplication.GetSnapshotOrdersAsync(new OrderInDto());
        var id = page.Items.First().Id;
        var detail = await _snapshotApplication.GetSnapshotOrderDetailAsync(id);
        detail.Id.ShouldBe(id);
        detail.IndustryName.ShouldNotBeNull();
    }

    [Theory]
    [InlineData(2, 2)]
    [InlineData(12, 12)]
    public async Task GetRedPackDateAsync(int count, int exp)
    {
        var items = await _snapshotApplication.GetRedPackDateAsync(new RedPackDateInDto { Count = count });
        items.Count.ShouldNotBe(0, "0数据");
        items.Count.ShouldBe(exp, $"应该:{exp}, 实际 {items.Count}");
    }

    [Theory]
    [InlineData(ERedPackPickupStatus.Unreceived)]
    [InlineData(ERedPackPickupStatus.Received)]
    public async Task GetRedPacksAsync(ERedPackPickupStatus status)
    {
        var page = await _snapshotApplication.GetRedPacksAsync(new RedPacksInDto { Status = status });
        page.Total.ShouldNotBe(0, "数据不应该为空");
    }

    [Fact]
    public async Task GetBulletinsDetail_Test()
    {
        var detail = await _snapshotApplication.GetBulletinsDetailAsync("08dc788f-20f4-4bf1-83d3-b5a8a4f395b0");
        detail.Id.ShouldNotBeNullOrEmpty();
        detail.Title.ShouldNotBeNullOrEmpty();
        detail.Content.ShouldNotBeNullOrEmpty();
    }

    [Fact]
    public async Task InitRedPackDataAsync()
    {
        for (int i = 0;i < 12;i++)
        {
            var now = DateTime.Now;
            var entity = new RedPackRecord
            {
                OrderId = "111111111",
                Amount = 10 * 1000,
                CreationTime = new DateTime(2024, i + 1, 02, now.Hour, now.Minute, now.Second),
                WXOpenId = "测试生成的OpenId",
                PickupStatus = ERedPackPickupStatus.Received,
            };
            await _redPackRecordRepository.AddAsync(entity);
        }
    }

    /// <summary>
    /// 测试行业
    /// 测试添加数据是否和获取的数据一致
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task Industry_Test()
    {
        var industry = new AddIndustryDto
        {
            Name = "测试行业",
            TitleSuffix = "测试标题",
            ApproveOrgId = "测试审批部门Id",
            ApproveOrgName = "测试审批部门名字",
            AcceptType = "测试受理类型",
            AcceptTypeCode = "测试受理类型代码",
            CitizenReadPackAmount = 100,
            GuiderReadPackAmount = 200,
            IsEnable = true,
            Files = new List<IndustryFileDto>
            {
                new IndustryFileDto
                {
                    Name = "测试文件" + DateTime.Now.ToShortDateString(),
                    Path = "测试文件地址" + DateTime.Now.ToShortTimeString(),
                    FileName ="测试文件" + DateTime.Now.ToShortDateString()  + ".doc",
                    AdditionId = DateTime.Now.ToLongDateString()
                }
            }
        };
        var industryId = await _industryApplication.AddIndustryAsync(industry, CancellationToken.None);
        var pageDto = await _snapshotApplication.GetIndustryBaseAsync(industryId, CancellationToken.None);
        try
        {
            pageDto.ShouldNotBeNull();
            pageDto.Files.ShouldNotBeNull();
            foreach (var file in pageDto.Files)
            {
                file.Id.ShouldNotBeNullOrEmpty();
                file.Name.ShouldNotBeNullOrEmpty();
                file.Path.ShouldNotBeNullOrEmpty();
                file.FileName.ShouldNotBeNullOrEmpty();
                file.AdditionId.ShouldNotBeNullOrEmpty();
                file.Key.ShouldBe(industryId);
            }
            pageDto.Workplace.ShouldNotBeNull();
            pageDto.WorkArea.ShouldNotBeNull();
        }
        catch (Exception e)
        {
            // ignore
        }
        finally
        {
            await _industryRepository.Removeable().Where(m => m.Id == industryId).ExecuteCommandAsync();
            await _fileRepository.Removeable().Where(m => m.Id == pageDto.Files.First().Id).ExecuteCommandAsync();
        }
    }
}
