﻿using Hotline.Caching.Interfaces;
using Hotline.SeedData;
using Hotline.Settings;
using Hotline.Share.Tools;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace Hotline.Caching.Services
{
    public class SysDicDataCacheManager : ISystemDicDataCacheManager, IScopeDependency
    {
        private readonly IRepository<SystemDicData> _sysDicDataRepository;
        private readonly IRepository<SystemDicType> _systemDicTypeRepository;
        private readonly ITypedCache<IReadOnlyList<SystemDicData>> _cacheSysDicData;

        public SysDicDataCacheManager(IRepository<SystemDicData> sysDicDataRepository, ITypedCache<IReadOnlyList<SystemDicData>> cacheSysDicData, IRepository<SystemDicType> systemDicTypeRepository)
        {
            _sysDicDataRepository = sysDicDataRepository;
            _cacheSysDicData = cacheSysDicData;
            _systemDicTypeRepository = systemDicTypeRepository;
        }


        public IReadOnlyList<SystemDicData> GetSysDicDataCache(string code)
        {
            var sysDicDataList = _cacheSysDicData.GetOrSet(code, k =>
            {
                return _sysDicDataRepository.Queryable().Where(x => x.DicTypeCode == code && x.IsShow == true).OrderBy(x => x.Sort).ToTreeAsync(x => x.Children, it => it.ParentId, "").GetAwaiter().GetResult();
            });
            return sysDicDataList;
        }

        public IReadOnlyList<SystemDicData> GetOrAdd(string code)
        {
            var items = GetSysDicDataCache(code);
            if (items.NotNullOrEmpty()) return items;

            var any = _sysDicDataRepository.Queryable().Where(m => m.DicTypeCode == code).Any();
            if (any) return new List<SystemDicData>();

            var sysDicType = _systemDicTypeRepository.Get(x => x.DicTypeCode == code);
            var seedData = new SystemDicDataSeedData();
            if (sysDicType == null)
            {
                sysDicType = seedData.GetType(code);
                sysDicType.Id = _systemDicTypeRepository.AddAsync(sysDicType).GetAwaiter().GetResult();
            }
            var sysDicDatas = seedData.GetData(code);
            AssignChilderDicType(sysDicDatas.ToList(), code, sysDicType.Id);

            RemoveSysDicDataCache(code);
            return GetSysDicDataCache(code);
        }

        private void AssignChilderDicType(IList<SystemDicData> sysDicDatas, string code, string dicTypeId, string parentId = "")
        {
            foreach (var item in sysDicDatas)
            {
                item.DicTypeCode = code;
                item.DicTypeId = dicTypeId;
                item.ParentId = parentId;
                item.IsShow = true;
                _sysDicDataRepository.AddAsync(item).GetAwaiter().GetResult();
                if (item.Children.NotNullOrEmpty()) AssignChilderDicType(item.Children, code, dicTypeId, item.Id);
            }
        }

        public IReadOnlyList<SystemDicData> GetVisitSatisfaction()
            => GetSysDicDataCache(SysDicTypeConsts.VisitSatisfaction);

        public IReadOnlyList<SystemDicData> AcceptType => GetSysDicDataCache(SysDicTypeConsts.AcceptType);

        public IReadOnlyList<SystemDicData> LeaderSMS => GetSysDicDataCache(SysDicTypeConsts.LeaderSMS);

        public IReadOnlyList<SystemDicData> CallForwardingSource => GetSysDicDataCache(SysDicTypeConsts.CallForwardingSource);

        /// <summary>
        /// 工作场所
        /// </summary>
        public IReadOnlyList<SystemDicData> Workplace => GetOrAdd(SysDicTypeConsts.Workplace);

        /// <summary>
        /// 作业区域
        /// </summary>
        public IReadOnlyList<SystemDicData> WorkArea => GetOrAdd(SysDicTypeConsts.WorkArea);

        public IReadOnlyList<SystemDicData> VisitMananer => GetSysDicDataCache(SysDicTypeConsts.VisitManner);
        public IReadOnlyList<SystemDicData> SourceChannel => GetSysDicDataCache(SysDicTypeConsts.SourceChannel);

        public void RemoveSysDicDataCache(string code)
        {
            _cacheSysDicData.Remove(code);
        }
    }
}
