﻿using Hotline.Caching.Interfaces;
using Hotline.Settings;
using Hotline.Share.Tools;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Caching.Services
{
    public class SystemSettingCacheManager : ISystemSettingCacheManager, IScopeDependency
    {

        private readonly ILogger<SystemSettingCacheManager> _logger;
        private readonly ITypedCache<SystemSetting> _cacheSystemSetting;
        private readonly IRepository<SystemSetting> _systemSettingRepository;

        public SystemSettingCacheManager(ITypedCache<SystemSetting> cacheSystemSetting, IRepository<SystemSetting> systemSettingRepository, ILogger<SystemSettingCacheManager> logger)
        {
            _cacheSystemSetting = cacheSystemSetting;
            _systemSettingRepository = systemSettingRepository;
            _logger = logger;
        }

        public SystemSetting? GetSetting(string code)
        {
            var setting = _cacheSystemSetting.GetOrSet(code, x =>
            {
                var dbsetting = _systemSettingRepository.GetAsync(d => d.Code == code).GetAwaiter().GetResult();
                if (dbsetting == null)
                    throw new UserFriendlyException($"{code} 无效系统设置");
                return dbsetting;
            });
            return setting;
        }

        public void DelSystemSetting(string code)
        {
            _cacheSystemSetting.Remove(code);
        }

        public T GetOrDefault<T>(string? id, string code, string name, T defaultValue, string remark) 
        {
            var value = defaultValue.ToString();
            try
            {
                try
                {
                    value = GetSetting(code)?.SettingValue[0];
                    if (value == null) return defaultValue;
                }
                catch (UserFriendlyException e)
                {
                    if (e.Message.Contains("无效系统设置"))
                    {
                        var entity = new SystemSetting
                        {
                            Code = code,
                            SettingName = name,
                            SettingValue = [defaultValue.ToString()],
                            Remark = remark,
                            CreatorName = "自动创建",
                        };
                        if (id.NotNullOrEmpty())
                            entity.Id = id;
                        _systemSettingRepository.AddAsync(entity).GetAwaiter().GetResult();
                    }
                }

                try
                {
                    return (T)Convert.ChangeType(value, typeof(T));
                }
                catch (InvalidCastException e)
                {
                    _logger.LogError($"无法将 '{value}' 转换为类型 {typeof(T).Name}！已返回默认值 {defaultValue}. {e.Message}");
                }
            }
            catch (Exception e)
            {
                _logger.LogError(e.Message + e.StackTrace);
            }
            return defaultValue;
        }

        public int EffectiveTimes
            => int.Parse(GetSetting(SettingConstants.EffectiveTimes)?.SettingValue[0]);

        public int ConnectByeTimes
            => int.Parse(GetSetting(SettingConstants.ConnectByeTimes)?.SettingValue[0]);

        public int NoConnectByeTimes => int.Parse(GetSetting(SettingConstants.NoConnectByeTimes)?.SettingValue[0]);

        public int CallInOverConnRingTime => int.Parse(GetSetting(SettingConstants.CallInOverConnRingTime)?.SettingValue[0]);

        public int SeatChaoTime => int.Parse(GetSetting(SettingConstants.SeatChaoTime)?.SettingValue[0]);

        public int RingTimes => int.Parse(GetSetting(SettingConstants.RingTimes)?.SettingValue[0]);

        public string RecordPrefix => GetSetting(SettingConstants.RecordPrefix)?.SettingValue[0].Trim().ToString();

        public string DefaultVisitEmployeeId => GetSetting(SettingConstants.DefaultVisitEmployeeId)?.SettingValue[0].Trim().ToString();

        public int FixedQueryCount => int.Parse(GetSetting(SettingConstants.FixedQueryCount)?.SettingValue[0]);

        /// <summary>
        /// 保存回访详情时发送延迟消息同步通话记录
        /// 如果回访通话记录有多条, 需要关联通话时长最长的那条
        /// </summary>
        public int VisitCallDelaySecond => 
            GetOrDefault(null, SettingConstants.VisitCallDelaySecond, "回访通话记录同步延时时间(秒)", 60, "保存回访详情时发送延迟消息同步通话记录,如果回访通话记录有多条, 需要关联通话时长最长的那条");

        public bool AutomaticPublishOrder =>
            GetOrDefault(null, SettingConstants.AutomaticPublishOrder, "自动发布中心直办归档工单", false, "用于中心直办件归档后默认自动发布, 开启就自动发布.true 或者 false");

        /// <summary>
        /// 取消发布功能总开关
        /// </summary>
        public bool CancelPublishOrderEnabled =>
            GetOrDefault(null, SettingConstants.CancelPublishOrderEnabled, "取消发布功能开关", false, "取消发布功能总开关, 关闭后取消发布功能的所有代码都不会执行.true 或者 false");

        /// <summary>
        /// 发送回访短信后等候市民回复短信的时间(秒)
        /// 如果超过这个时间就更新回访为默认满意
        /// </summary>
        public int DefaultVisitSmsDelaySecond =>
            GetOrDefault<int>("08dc0681-a6d2-4ce7-877d-db65f846d523", SettingConstants.DefaultVisitSmsDelaySecond, "发送回访短信后等候市民回复短信的时间(秒)", 172800,
                "发送回访短信后等候市民回复短信的时间(秒), 如果超过这个时间就更新回访为默认满意. 默认 172800.");

        /// <summary>
        /// 通话记录同步时不推送省上的时间(默认:2024/11/19 18:08)
        /// </summary>
        public DateTime CallSyncUnPushDateTime =>
            GetOrDefault("08dc9bed-41da-4074-8289-e753e8fa8ea4", SettingConstants.CallSyncUnPushDateTime, "通话记录同步时不推送省上的时间(默认:2024/11/19 18:08)",
                DateTime.Parse("2024/11/19 18:08:00"), "通话记录同步时不推送省上的时间(默认:2024/11/19 18:08), 在兴唐数据库 callstarttime 字段时间小于设置 (默认:2024/11/19 18:08) 的时间都不会推送给省上");

        /// <summary>
        /// 即将超期列表调用方法的版本(有优化办法和老版本)
        /// </summary>
        public int GetAboutToExpireVersion =>
            GetOrDefault("08dc39bd-0ead-477c-87bc-97252dd8fb26", SettingConstants.GetAboutToExpireVersion, "前调用后端超期列表时后端控制调用优化的代码还是老代码", 0, "前调用后端超期列表时后端控制调用优化的代码还是老代码, 目前只有一个优化版本. 该值为 1 调用优化过的代码.");

        /// <summary>
        /// 工单开始办理如果获取的通话记录是呼出并且录音文件是空就不推送通话记录
        /// </summary>
        public bool OrderStartHandlerPushCallIsNull =>
            GetOrDefault("08dc9a54-d6ea-4757-8934-daff80efb86e", SettingConstants.OrderStartHandlerPushCallIsNull, "工单开始办理如果获取的通话记录是呼出并且录音文件是空就不推送通话记录", false, "工单开始办理如果获取的通话记录是呼出并且录音文件是空就不推送通话记录. true 不推通话记录, false 老逻辑继续推送");

        /// <summary>
        /// 微信小程序Secret
        /// </summary>
        public string WxOpenAppSecret =>
            GetOrDefault<string>("08dccbbc-4121-4f9c-87ba-ecdbce0bc15f", SettingConstants.WxOpenAppSecret, "微信小程序Secret", "bb29f684a9ea6b9d136062d68e867831", "微信小程序Secret");

        /// <summary>
        /// 微信小程序OpenId
        /// </summary>
        public string WxOpenAppId =>
            GetOrDefault("08dccbbc-090a-4446-842f-be95fc74c95e", SettingConstants.WxOpenAppId, "微信小程序OpenId", "wx22e6092dd504b567", "微信小程序OpenId");

        /// <summary>
        /// 文件服务器地址
        /// </summary>
        public string FileServerUrl =>
            GetOrDefault("08dc9a54-d6b1-46c9-8b9b-201025f629a1", SettingConstants.FileServerUrl, "文件服务器地址", "http://110.188.24.28:50120", "文件服务器地址");

        public string FileDownloadApi =>
            GetOrDefault("08dc9a54-d62e-4cdf-84ee-6c95859eec77", SettingConstants.FileDownloadApi, "文件服务器下载接口", "/file/downloadfile?Source=hotline&Id=", "文件服务器下载接口");

        /// <summary>
        /// 小程序Banner集合
        /// </summary>
        public string AppBanner =>
            GetOrDefault("08dcfe0a-e987-4981-8918-c272baaa4b04", SettingConstants.AppBanner, "小程序Bnner集合, 多个使用 | 分隔, 只写图片ID即可", "08dd136c-aed1-400a-8123-222987d4b087|08dd136c-aed1-400a-8123-222987d4b087", "小程序Bnner集合, 多个使用 | 分隔, 只写图片ID即可");
    }
}
